@@ -3,6 +3,7 @@ import sys
 
 from localstack import config
 from localstack.constants import TRUE_STRINGS
+from localstack.utils.aws.request_context import patch_request_handling
 from localstack.utils.bootstrap import ENV_SCRIPT_STARTING_DOCKER
 
 # Note: make sure not to add any additional imports at the global scope here!
@@ -47,7 +48,7 @@ def do_register_localstack_plugins():
             start_sts,
         )
         from localstack.services.kinesis import kinesis_listener, kinesis_starter
-        from localstack.services.kms import kms_starter
+        from localstack.services.kms import kms_listener, kms_starter
         from localstack.services.logs import logs_listener, logs_starter
         from localstack.services.plugins import Plugin, register_plugin
         from localstack.services.redshift import redshift_starter
@@ -126,7 +127,9 @@ def do_register_localstack_plugins():
             )
         )
 
-        register_plugin(Plugin("kms", start=kms_starter.start_kms, priority=10))
+        register_plugin(
+            Plugin("kms", start=kms_starter.start_kms, listener=kms_listener.UPDATE_KMS)
+        )
 
         register_plugin(Plugin("lambda", start=start_lambda))
 
@@ -228,6 +231,9 @@ def do_register_localstack_plugins():
 
         register_plugin(Plugin("support", start=support_starter.start_support))
 
+        # apply patches
+        patch_request_handling()
+
     except Exception as e:
         if not os.environ.get(ENV_SCRIPT_STARTING_DOCKER):
             print("Unable to register plugins: %s" % e)
