@@ -10,7 +10,7 @@ from moto.iam.models import Role as MotoRole
 from moto.s3.models import FakeBucket
 from moto.sqs.models import Queue as MotoQueue
 
-from localstack.constants import AWS_REGION_US_EAST_1, LOCALHOST
+from localstack.constants import AWS_REGION_US_EAST_1, S3_VIRTUAL_HOSTNAME
 from localstack.services.awslambda.lambda_api import (
     LAMBDA_POLICY_NAME_PATTERN,
     get_handler_file_from_name,
@@ -294,11 +294,24 @@ class EventBus(GenericBaseModel):
         return "AWS::Events::EventBus"
 
     def fetch_state(self, stack_name, resources):
-        event_bus_arn = self.physical_resource_id
-        if not event_bus_arn:
-            return None
+        event_bus_name = self.props.get("Name")
         client = aws_stack.connect_to_service("events")
-        return client.describe_event_bus(Name=event_bus_arn.split("/")[1])
+        return client.describe_event_bus(Name=event_bus_name)
+
+    def get_cfn_attribute(self, attribute_name):
+        props = self.props
+        if attribute_name in REF_ATTRS + ["Name"]:
+            return props.get("Name")
+        if attribute_name == "Arn":
+            return props.get("Arn")
+        return super(EventBus, self).get_cfn_attribute(attribute_name)
+
+    @classmethod
+    def get_deploy_templates(cls):
+        return {
+            "create": {"function": "create_event_bus", "parameters": {"Name": "Name"}},
+            "delete": {"function": "delete_event_bus", "parameters": {"Name": "Name"}},
+        }
 
 
 class LogsLogGroup(GenericBaseModel):
@@ -993,7 +1006,7 @@ class IAMManagedPolicy(GenericBaseModel):
         return "AWS::IAM::ManagedPolicy"
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
-        return aws_stack.role_arn(self.props["ManagedPolicyName"])
+        return aws_stack.policy_arn(self.props["ManagedPolicyName"])
 
     def fetch_state(self, stack_name, resources):
         return IAMPolicy.get_policy_state(self, stack_name, resources, managed_policy=True)
@@ -1021,6 +1034,44 @@ class IAMManagedPolicy(GenericBaseModel):
         return {"create": {"function": _create}}
 
 
+class IAMUser(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return "AWS::IAM::User"
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get("UserName")
+
+    def get_resource_name(self):
+        return self.props.get("UserName")
+
+    def fetch_state(self, stack_name, resources):
+        user_name = self.resolve_refs_recursively(stack_name, self.props.get("UserName"), resources)
+        return aws_stack.connect_to_service("iam").get_user(UserName=user_name)["User"]
+
+    def update_resource(self, new_resource, stack_name, resources):
+        props = new_resource["Properties"]
+        client = aws_stack.connect_to_service("iam")
+        return client.update_user(
+            UserName=props.get("UserName"),
+            NewPath=props.get("NewPath") or "",
+            NewUserName=props.get("NewUserName") or "",
+        )
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            "create": {
+                "function": "create_user",
+                "parameters": ["Path", "UserName", "PermissionsBoundary", "Tags"],
+            },
+            "delete": {
+                "function": "delete_user",
+                "parameters": ["UserName"],
+            },
+        }
+
+
 class GatewayResponse(GenericBaseModel):
     @staticmethod
     def cloudformation_type():
@@ -1161,6 +1212,33 @@ class GatewayResource(GenericBaseModel):
         result = list(filter(lambda res: res["path"] == path, api_resources))
         return result[0] if result else None
 
+    @staticmethod
+    def get_deploy_templates():
+        def get_apigw_resource_params(params, **kwargs):
+            result = {
+                "restApiId": params.get("RestApiId"),
+                "pathPart": params.get("PathPart"),
+                "parentId": params.get("ParentId"),
+            }
+            if not result.get("parentId"):
+                # get root resource id
+                apigw = aws_stack.connect_to_service("apigateway")
+                resources = apigw.get_resources(restApiId=result["restApiId"])["items"]
+                root_resource = ([r for r in resources if r["path"] == "/"] or [None])[0]
+                if not root_resource:
+                    raise Exception(
+                        "Unable to find root resource for REST API %s" % result["restApiId"]
+                    )
+                result["parentId"] = root_resource["id"]
+            return result
+
+        return {
+            "create": {
+                "function": "create_resource",
+                "parameters": get_apigw_resource_params,
+            }
+        }
+
 
 class GatewayMethod(GenericBaseModel):
     @staticmethod
@@ -1486,7 +1564,7 @@ class S3Bucket(GenericBaseModel, FakeBucket):
 
     def fetch_state(self, stack_name, resources):
         props = self.props
-        bucket_name = props.get("BucketName") or self.resource_id
+        bucket_name = self._get_bucket_name()
         bucket_name = self.resolve_refs_recursively(stack_name, bucket_name, resources)
         bucket_name = self.normalize_bucket_name(bucket_name)
         s3_client = aws_stack.connect_to_service("s3")
@@ -1506,9 +1584,13 @@ class S3Bucket(GenericBaseModel, FakeBucket):
 
     def get_cfn_attribute(self, attribute_name):
         if attribute_name in ["DomainName", "RegionalDomainName"]:
-            return LOCALHOST
+            bucket_name = self._get_bucket_name()
+            return "%s.%s" % (bucket_name, S3_VIRTUAL_HOSTNAME)
         return super(S3Bucket, self).get_cfn_attribute(attribute_name)
 
+    def _get_bucket_name(self):
+        return self.props.get("BucketName") or self.resource_id
+
 
 class S3BucketPolicy(GenericBaseModel):
     @staticmethod
@@ -1668,14 +1750,34 @@ class SNSTopic(GenericBaseModel):
 
     @staticmethod
     def get_deploy_templates():
+        def _create_params(params, *args, **kwargs):
+            attributes = {}
+            dedup = params.get("ContentBasedDeduplication")
+            display_name = params.get("DisplayName")
+            fifo_topic = params.get("FifoTopic")
+            kms_master_key = params.get("KmsMasterKeyId")
+            tags = params.get("Tags") or []
+            topic_name = params.get("TopicName")
+            if dedup is not None:
+                attributes["ContentBasedDeduplication"] = dedup
+            if display_name:
+                attributes["DisplayName"] = display_name
+            if fifo_topic is not None:
+                attributes["FifoTopic"] = fifo_topic
+            if kms_master_key:
+                attributes["KmsMasterKeyId"] = kms_master_key
+            result = {"Name": topic_name, "Attributes": attributes, "Tags": tags}
+            return result
+
         def _topic_arn(params, resources, resource_id, **kwargs):
             resource = SNSTopic(resources[resource_id])
             return resource.physical_resource_id or resource.get_physical_resource_id()
 
         return {
+            # TODO: add second creation function to add topic subscriptions
             "create": {
                 "function": "create_topic",
-                "parameters": {"Name": "TopicName", "Tags": "Tags"},
+                "parameters": _create_params,
             },
             "delete": {
                 "function": "delete_topic",
@@ -1819,6 +1921,7 @@ class SSMParameter(GenericBaseModel):
                         "Tier",
                     ),
                 ),
+                "types": {"Value": str},
             }
         }
 
