@@ -30,10 +30,15 @@ from localstack.services.plugins import (
     record_service_health,
     wait_for_infra_shutdown,
 )
-from localstack.utils import common, config_listener, persistence
+from localstack.utils import analytics, common, config_listener, persistence
 from localstack.utils.analytics import event_publisher
-from localstack.utils.analytics.profiler import log_duration
-from localstack.utils.bootstrap import canonicalize_api_names, in_ci, load_plugins, setup_logging
+from localstack.utils.bootstrap import (
+    canonicalize_api_names,
+    in_ci,
+    load_plugins,
+    log_duration,
+    setup_logging,
+)
 from localstack.utils.cli import print_version
 from localstack.utils.common import (
     TMP_THREADS,
@@ -362,6 +367,7 @@ def stop_infra():
     common.INFRA_STOPPED = True
 
     event_publisher.fire_event(event_publisher.EVENT_STOP_INFRA)
+    analytics.log.event("infra_stop")
 
     try:
         generic_proxy.QUIET = True
@@ -487,6 +493,7 @@ def do_start_infra(asynchronous, apis, is_in_docker):
 
     # prepare APIs
     apis = canonicalize_api_names(apis)
+    analytics.log.event("infra_start", apis=apis)
 
     @log_duration()
     def prepare_environment():
@@ -543,5 +550,6 @@ def do_start_infra(asynchronous, apis, is_in_docker):
     sys.stdout.flush()
 
     INFRA_READY.set()
+    analytics.log.event("infra_ready")
 
     return thread
