@@ -8,6 +8,7 @@ import shutil
 import sys
 import tempfile
 import time
+from pathlib import Path
 
 import requests
 
@@ -28,11 +29,10 @@ from localstack.constants import (
     LOCALSTACK_MAVEN_VERSION,
     MODULE_MAIN_PATH,
     STS_JAR_URL,
-    THUNDRA_APIKEY,
-    THUNDRA_JAVA_AGENT_JAR_NAME,
-    THUNDRA_JAVA_AGENT_VERSION,
 )
+from localstack.plugin import thundra
 from localstack.utils import bootstrap
+from localstack.utils.docker import DOCKER_CLIENT
 
 if __name__ == "__main__":
     bootstrap.bootstrap_installation()
@@ -65,7 +65,6 @@ INSTALL_DIR_STEPFUNCTIONS = "%s/stepfunctions" % INSTALL_DIR_INFRA
 INSTALL_DIR_KMS = "%s/kms" % INSTALL_DIR_INFRA
 INSTALL_DIR_ELASTICMQ = "%s/elasticmq" % INSTALL_DIR_INFRA
 INSTALL_PATH_LOCALSTACK_FAT_JAR = "%s/localstack-utils-fat.jar" % INSTALL_DIR_INFRA
-INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR = "%s/%s" % (config.TMP_FOLDER, THUNDRA_JAVA_AGENT_JAR_NAME)
 INSTALL_PATH_DDB_JAR = os.path.join(INSTALL_DIR_DDB, "DynamoDBLocal.jar")
 INSTALL_PATH_KCL_JAR = os.path.join(INSTALL_DIR_KCL, "aws-java-sdk-sts.jar")
 INSTALL_PATH_STEPFUNCTIONS_JAR = os.path.join(INSTALL_DIR_STEPFUNCTIONS, "StepFunctionsLocal.jar")
@@ -77,10 +76,7 @@ URL_LOCALSTACK_FAT_JAR = (
     "https://repo1.maven.org/maven2/"
     + "cloud/localstack/localstack-utils/{v}/localstack-utils-{v}-fat.jar"
 ).format(v=LOCALSTACK_MAVEN_VERSION)
-THUNDRA_JAVA_AGENT_JAR = (
-    "https://repo.thundra.io/service/local/artifact/maven/redirect?"
-    + "r=thundra-releases&g=io.thundra.agent&a=thundra-agent-lambda-bootstrap&v={v}"
-).format(v=THUNDRA_JAVA_AGENT_VERSION)
+
 MARKER_FILE_LIGHT_VERSION = "%s/.light-version" % INSTALL_DIR_INFRA
 IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local"
 ARTIFACTS_REPO = "https://github.com/localstack/localstack-artifacts"
@@ -93,7 +89,7 @@ SFN_PATCH_CLASS_URL = "%s/raw/master/stepfunctions-local-patch/%s" % (
 )
 
 # kinesis-mock version
-KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.1.7"
+KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.1.8"
 KINESIS_MOCK_RELEASE_URL = (
     "https://api.github.com/repos/etspaceman/kinesis-mock/releases/tags/" + KINESIS_MOCK_VERSION
 )
@@ -296,20 +292,24 @@ def install_stepfunctions_local():
         # TODO: works only when running on the host, outside of Docker -> add a fallback if running in Docker?
         log_install_msg("Step Functions")
         mkdir(INSTALL_DIR_STEPFUNCTIONS)
-        run("{dc} pull {img}".format(dc=config.DOCKER_CMD, img=IMAGE_NAME_SFN_LOCAL))
+        DOCKER_CLIENT.pull_image(IMAGE_NAME_SFN_LOCAL)
         docker_name = "tmp-ls-sfn"
-        run(
-            ("{dc} run --name={dn} --entrypoint= -d --rm {img} sleep 15").format(
-                dc=config.DOCKER_CMD, dn=docker_name, img=IMAGE_NAME_SFN_LOCAL
-            )
+        DOCKER_CLIENT.run_container(
+            IMAGE_NAME_SFN_LOCAL,
+            remove=True,
+            entrypoint="",
+            name=docker_name,
+            detach=True,
+            command=["sleep", "15"],
         )
         time.sleep(5)
-        run(
-            "{dc} cp {dn}:/home/stepfunctionslocal/ {tgt}".format(
-                dc=config.DOCKER_CMD, dn=docker_name, tgt=INSTALL_DIR_INFRA
-            )
+        DOCKER_CLIENT.copy_from_container(
+            docker_name, local_path=INSTALL_DIR_INFRA, container_path="/home/stepfunctionslocal/"
         )
-        run("mv %s/stepfunctionslocal/*.jar %s" % (INSTALL_DIR_INFRA, INSTALL_DIR_STEPFUNCTIONS))
+
+        path = Path(f"{INSTALL_DIR_INFRA}/stepfunctionslocal/")
+        for file in path.glob("*.jar"):
+            file.rename(Path(INSTALL_DIR_STEPFUNCTIONS) / file.name)
         rm_rf("%s/stepfunctionslocal" % INSTALL_DIR_INFRA)
     # apply patches
     patch_class_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, SFN_PATCH_CLASS)
@@ -379,13 +379,6 @@ def install_lambda_java_libs():
         download(URL_LOCALSTACK_FAT_JAR, INSTALL_PATH_LOCALSTACK_FAT_JAR)
 
 
-def install_thundra_java_agent():
-    # install Thundra Java agent JAR file
-    if not os.path.exists(INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR):
-        log_install_msg("Thundra Java agent", verbatim=True)
-        download(THUNDRA_JAVA_AGENT_JAR, INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR)
-
-
 def install_cloudformation_libs():
     from localstack.services.cloudformation import deployment_utils
 
@@ -410,11 +403,9 @@ def install_component(name):
 def install_components(names):
     parallelize(install_component, names)
     install_lambda_java_libs()
-    # If Thundra API key is specified,
-    # this means that Thundra integration will be used.
-    # So, install Thundra agent.
-    if THUNDRA_APIKEY:
-        install_thundra_java_agent()
+
+    # Init Thundra
+    thundra.init()
 
 
 def install_all_components():
