@@ -91,17 +91,18 @@ def patch_logs():
         return put_subscription_filter
 
     def put_log_events_model(self, log_group_name, log_stream_name, log_events, sequence_token):
-        self.lastIngestionTime = int(unix_time_millis())
-        self.storedBytes += sum([len(log_event["message"]) for log_event in log_events])
+        # TODO: call/patch upstream method here, instead of duplicating the code!
+        self.last_ingestion_time = int(unix_time_millis())
+        self.stored_bytes += sum([len(log_event["message"]) for log_event in log_events])
         events = [
-            logs_models.LogEvent(self.lastIngestionTime, log_event) for log_event in log_events
+            logs_models.LogEvent(self.last_ingestion_time, log_event) for log_event in log_events
         ]
         self.events += events
-        self.uploadSequenceToken += 1
+        self.upload_sequence_token += 1
 
         log_events = [
             {
-                "id": event.eventId,
+                "id": event.event_id,
                 "timestamp": event.timestamp,
                 "message": event.message,
             }
