@@ -1,4 +1,5 @@
 import base64
+import copy
 import json
 import logging
 import re
@@ -6,6 +7,7 @@ import traceback
 from typing import Optional
 from urllib.parse import urlparse
 
+import botocore
 from moto.cloudformation import parsing
 from moto.core import CloudFormationModel as MotoCloudFormationModel
 from moto.ec2.utils import generate_route_id
@@ -129,23 +131,6 @@ def get_ddb_kinesis_stream_specification(params, **kwargs):
     return args
 
 
-def get_apigw_resource_params(params, **kwargs):
-    result = {
-        "restApiId": params.get("RestApiId"),
-        "pathPart": params.get("PathPart"),
-        "parentId": params.get("ParentId"),
-    }
-    if not result.get("parentId"):
-        # get root resource id
-        apigw = aws_stack.connect_to_service("apigateway")
-        resources = apigw.get_resources(restApiId=result["restApiId"])["items"]
-        root_resource = ([r for r in resources if r["path"] == "/"] or [None])[0]
-        if not root_resource:
-            raise Exception("Unable to find root resource for REST API %s" % result["restApiId"])
-        result["parentId"] = root_resource["id"]
-    return result
-
-
 # maps resource types to functions and parameters for creation
 RESOURCE_TO_FUNCTION = {
     "S3::BucketPolicy": {
@@ -252,10 +237,6 @@ RESOURCE_TO_FUNCTION = {
             "parameters": {"TableName": "TableName"},
         },
     },
-    "Events::EventBus": {
-        "create": {"function": "create_event_bus", "parameters": {"Name": "Name"}},
-        "delete": {"function": "delete_event_bus", "parameters": {"Name": "Name"}},
-    },
     "IAM::Role": {
         "create": {
             "function": "create_role",
@@ -277,12 +258,6 @@ RESOURCE_TO_FUNCTION = {
         },
         "delete": {"function": "delete_role", "parameters": {"RoleName": "RoleName"}},
     },
-    "ApiGateway::Resource": {
-        "create": {
-            "function": "create_resource",
-            "parameters": get_apigw_resource_params,
-        }
-    },
     "ApiGateway::Method": {
         "create": {
             "function": "put_method",
@@ -1046,6 +1021,8 @@ def convert_data_types(func_details, params):
         if _type == bool:
             return _obj in ["True", "true", True]
         if _type == str:
+            if isinstance(_obj, bool):
+                return str(_obj).lower()
             return str(_obj)
         if _type == int:
             return int(_obj)
@@ -1298,15 +1275,12 @@ def configure_resource_via_sdk(
     if params is None:
         return
 
-    # convert refs and boolean strings
+    # convert refs
     for param_key, param_value in dict(params).items():
         if param_value is not None:
             param_value = params[param_key] = resolve_refs_recursively(
                 stack_name, param_value, resources
             )
-        # Convert to boolean (TODO: do this recursively?)
-        if str(param_value).lower() in ["true", "false"]:
-            params[param_key] = str(param_value).lower() == "true"
 
     # convert any moto account IDs (123456789012) in ARNs to our format (000000000000)
     params = fix_account_id_in_arns(params)
@@ -1318,13 +1292,26 @@ def configure_resource_via_sdk(
     # run pre-actions
     run_pre_create_actions(action_name, resource_id, resources, resource_type, stack_name, params)
 
+    # convert boolean strings
+    #  (TODO: we should find a more reliable mechanism than this opportunistic/probabilistic approach!)
+    params_before_conversion = copy.deepcopy(params)
+    for param_key, param_value in dict(params).items():
+        # Convert to boolean (TODO: do this recursively?)
+        if str(param_value).lower() in ["true", "false"]:
+            params[param_key] = str(param_value).lower() == "true"
+
     # invoke function
     try:
         LOG.debug(
             'Request for resource type "%s" in region %s: %s %s'
             % (resource_type, aws_stack.get_region(), func_details["function"], params)
         )
-        result = function(**params)
+        try:
+            result = function(**params)
+        except botocore.exceptions.ParamValidationError as e:
+            if "type: <class 'bool'>" not in str(e):
+                raise
+            result = function(**params_before_conversion)
     except Exception as e:
         if action_name == "delete" and check_not_found_exception(e, resource_type, resource):
             return
@@ -1506,6 +1493,25 @@ def run_post_create_actions(action_name, resource_id, resources, resource_type,
                 InstanceProfileName=resource_props["InstanceProfileName"],
                 RoleName=resource_props["Roles"][0],
             )
+    elif resource_type == "IAM::User":
+        iam = aws_stack.connect_to_service("iam")
+        username = resource_props.get("UserName")
+        for group in resource_props.get("Groups", []):
+            iam.add_user_to_group(UserName=username, GroupName=group)
+        for managed_policy in resource_props.get("ManagedPolicyArns", []):
+            iam.attach_user_policy(UserName=username, PolicyArn=managed_policy)
+        for policy in resource_props.get("Policies", []):
+            policy_doc = json.dumps(policy.get("PolicyDocument"))
+            iam.put_user_policy(
+                UserName=username, PolicyName=policy.get("PolicyName"), PolicyDocument=policy_doc
+            )
+        login_profile = resource_props.get("LoginProfile")
+        if login_profile:
+            iam.create_login_profile(
+                UserName=username,
+                Password=login_profile.get("Password"),
+                PasswordResetRequired=login_profile.get("PasswordResetRequired"),
+            )
 
 
 def get_action_name_for_resource_change(res_change):
@@ -1607,6 +1613,10 @@ def update_resource_details(stack, resource_id, details, action=None):
     if not resource or not details:
         return
 
+    # TODO: we need to rethink this method - this should be encapsulated in the resource model classes.
+    #   Also, instead of actively updating the PhysicalResourceId attributes below, they should be
+    #   determined and returned by the resource model classes upon request.
+
     resource_type = resource.get("Type") or ""
     resource_type = re.sub("^AWS::", "", resource_type)
     resource_props = resource.get("Properties", {})
@@ -1627,9 +1637,6 @@ def update_resource_details(stack, resource_id, details, action=None):
     if resource_type == "IAM::InstanceProfile":
         resource["PhysicalResourceId"] = details["InstanceProfile"]["InstanceProfileName"]
 
-    if resource_type == "Events::EventBus":
-        resource["PhysicalResourceId"] = details["EventBusArn"]
-
     if resource_type == "StepFunctions::Activity":
         resource["PhysicalResourceId"] = details["activityArn"]
 
