@@ -32,11 +32,11 @@ import dns.resolver
 import requests
 import six
 
+import localstack.utils.run
 from localstack import config
 from localstack.config import DEFAULT_ENCODING
 from localstack.constants import ENV_DEV
-from localstack.utils import bootstrap
-from localstack.utils.bootstrap import FuncThread
+from localstack.utils.run import FuncThread
 
 # arrays for temporary files and resources
 TMP_FILES = []
@@ -70,6 +70,9 @@ CACHE = {}
 # lock for creating certificate files
 SSL_CERT_LOCK = threading.RLock()
 
+# user of the currently running process
+CACHED_USER = None
+
 
 class Mock(object):
     """Dummy class that can be used for mocking custom attributes."""
@@ -195,7 +198,8 @@ class ShellCommandThread(FuncThread):
                             if self.outfile not in [None, os.devnull]:
                                 outstream.write(line)
                                 outstream.flush()
-                self.process.wait()
+                if self.process:
+                    self.process.wait()
             else:
                 self.process.communicate()
         except Exception as e:
@@ -491,10 +495,19 @@ def md5(string):
 
 
 def select_attributes(obj, attributes):
+    """Select a subset of attributes from the given dict (returns a copy)"""
     attributes = attributes if is_list_or_tuple(attributes) else [attributes]
     return dict([(k, v) for k, v in obj.items() if k in attributes])
 
 
+def remove_attributes(obj, attributes):
+    """Remove a set of attributes from the given dict (in-place)"""
+    attributes = attributes if is_list_or_tuple(attributes) else [attributes]
+    for attr in attributes:
+        obj.pop(attr, None)
+    return obj
+
+
 def is_list_or_tuple(obj):
     return isinstance(obj, (list, tuple))
 
@@ -503,47 +516,6 @@ def in_docker():
     return config.in_docker()
 
 
-def has_docker():
-    try:
-        run("docker ps")
-        return True
-    except Exception:
-        return False
-
-
-def get_docker_container_names():
-    return bootstrap.get_docker_container_names()
-
-
-def get_docker_image_names(strip_latest=True):
-    cmd = "%s images --format '{{.Repository}}:{{.Tag}}'" % config.DOCKER_CMD
-    try:
-        output = to_str(run(cmd))
-        image_names = re.split(r"\s+", output.strip().replace("\n", " "))
-        if strip_latest:
-            suffix = ":latest"
-            for image in list(image_names):
-                if image.endswith(suffix):
-                    image_names.append(image[: -len(suffix)])
-        return image_names
-    except Exception as e:
-        LOG.info('Unable to list Docker images via "%s": %s' % (cmd, e))
-        return []
-
-
-def rm_docker_container(container_name_or_id, check_existence=False, safe=False):
-    if not container_name_or_id:
-        return
-    if check_existence and container_name_or_id not in get_docker_container_names():
-        # TODO: check names as well as container IDs!
-        return
-    try:
-        run("%s rm -f %s" % (config.DOCKER_CMD, container_name_or_id), print_error=False)
-    except Exception:
-        if not safe:
-            raise
-
-
 def path_from_url(url):
     return "/%s" % str(url).partition("://")[2].partition("/")[2] if "://" in url else url
 
@@ -722,18 +694,18 @@ def poll_condition(condition, timeout: float = None, interval: float = 0.5) -> b
     return True
 
 
-def merge_recursive(source, destination, none_values=[None]):
+def merge_recursive(source, destination, none_values=[None], overwrite=False):
     for key, value in source.items():
         if isinstance(value, dict):
             # get node or create one
             node = destination.setdefault(key, {})
-            merge_recursive(value, node)
+            merge_recursive(value, node, none_values=none_values, overwrite=overwrite)
         else:
             if not isinstance(destination, dict):
                 LOG.warning(
                     "Destination for merging %s=%s is not dict: %s", key, value, destination
                 )
-            if destination.get(key) in none_values:
+            if overwrite or destination.get(key) in none_values:
                 destination[key] = value
     return destination
 
@@ -882,7 +854,10 @@ def rm_rf(path):
         except Exception:
             pass
     # Make sure all files are writeable and dirs executable to remove
-    chmod_r(path, 0o777)
+    try:
+        chmod_r(path, 0o777)
+    except PermissionError:
+        pass  # todo log
     # check if the file is either a normal file, or, e.g., a fifo
     exists_but_non_dir = os.path.exists(path) and not os.path.isdir(path)
     if os.path.isfile(path) or exists_but_non_dir:
@@ -891,20 +866,53 @@ def rm_rf(path):
         shutil.rmtree(path)
 
 
-def cp_r(src, dst, rm_dest_on_conflict=False):
+def cp_r(src, dst, rm_dest_on_conflict=False, ignore_copystat_errors=False, **kwargs):
     """Recursively copies file/directory"""
-    if os.path.isfile(src):
-        return shutil.copy(src, dst)
-    kwargs = {}
-    if "dirs_exist_ok" in inspect.getfullargspec(shutil.copytree).args:
-        kwargs["dirs_exist_ok"] = True
+    # attention: this patch is not threadsafe
+    copystat_orig = shutil.copystat
+    if ignore_copystat_errors:
+
+        def _copystat(*args, **kwargs):
+            try:
+                return copystat_orig(*args, **kwargs)
+            except Exception:
+                pass
+
+        shutil.copystat = _copystat
     try:
-        return shutil.copytree(src, dst, **kwargs)
-    except FileExistsError:
-        if rm_dest_on_conflict:
-            rm_rf(dst)
+        if os.path.isfile(src):
+            if os.path.isdir(dst):
+                dst = os.path.join(dst, os.path.basename(src))
+            return shutil.copyfile(src, dst)
+        if "dirs_exist_ok" in inspect.getfullargspec(shutil.copytree).args:
+            kwargs["dirs_exist_ok"] = True
+        try:
             return shutil.copytree(src, dst, **kwargs)
+        except FileExistsError:
+            if rm_dest_on_conflict:
+                rm_rf(dst)
+                return shutil.copytree(src, dst, **kwargs)
+            raise
+    except Exception as e:
+
+        def _info(_path):
+            return "%s (file=%s, symlink=%s)" % (
+                _path,
+                os.path.isfile(_path),
+                os.path.islink(_path),
+            )
+
+        LOG.debug(
+            "Error copying files from %s to %s: %s"
+            % (
+                _info(src),
+                _info(dst),
+                e,
+            )
+        )
         raise
+    finally:
+        shutil.copystat = copystat_orig
 
 
 def disk_usage(path):
@@ -1017,11 +1025,11 @@ def is_number(s):
 
 
 def is_mac_os():
-    return bootstrap.is_mac_os()
+    return localstack.utils.run.is_mac_os()
 
 
 def is_linux():
-    return bootstrap.is_linux()
+    return localstack.utils.run.is_linux()
 
 
 def is_windows():
@@ -1328,7 +1336,7 @@ def cleanup_tmp_files():
     del TMP_FILES[:]
 
 
-def new_tmp_file():
+def new_tmp_file() -> str:
     """Return a path to a new temporary file."""
     tmp_file, tmp_path = tempfile.mkstemp()
     os.close(tmp_file)
@@ -1403,8 +1411,17 @@ def untar(path, target_dir):
 
 
 def is_root():
-    out = run("whoami").strip()
-    return out == "root"
+    return get_os_user() == "root"
+
+
+def get_os_user():
+    global CACHED_USER
+    if not CACHED_USER:
+        # TODO: using getpass.getuser() seems to be reporting a different/invalid user in Docker/MacOS
+        # import getpass
+        # CACHED_USER = getpass.getuser()
+        CACHED_USER = run("whoami").strip()
+    return CACHED_USER
 
 
 def cleanup_resources():
@@ -1608,14 +1625,14 @@ def do_run(cmd: str, run_cmd: Callable, cache_duration_secs: int):
 
 def run(cmd, cache_duration_secs=0, **kwargs):
     def run_cmd():
-        return bootstrap.run(cmd, **kwargs)
+        return localstack.utils.run.run(cmd, **kwargs)
 
     return do_run(cmd, run_cmd, cache_duration_secs)
 
 
 def safe_run(cmd: List[str], cache_duration_secs=0, **kwargs) -> Union[str, subprocess.Popen]:
     def run_cmd():
-        return bootstrap.run(cmd, shell=False, **kwargs)
+        return localstack.utils.run.run(cmd, shell=False, **kwargs)
 
     return do_run(" ".join(cmd), run_cmd, cache_duration_secs)
 
